from ImageSeeker.utils.config import configureData
from tensorflow.keras.preprocessing.image import ImageDataGenerator
from tensorflow.keras.preprocessing import image
import numpy as np

config = configureData()  #Defining the configureData object

def train_valid_generator():

    '''
    This function generates train & valid data from images path,
    it also performs data augmentation.
    :return object of data:
    '''

    if config['AUGMENTATION'] == 'True':
        print("Augmetation applied!")
        train_datagen = ImageDataGenerator(rescale=1. / 255,
                                           shear_range=0.2,
                                           zoom_range=0.2,
                                           horizontal_flip=True)

        valid_datagen = ImageDataGenerator(rescale=1. / 255)

        training_set = train_datagen.flow_from_directory(
            directory= config['TRAIN_DATA_DIR'],
            target_size=config['IMAGE_SIZE'][:-1],
            batch_size=config['BATCH_SIZE'],
            class_mode='categorical')

        valid_set = valid_datagen.flow_from_directory(
            directory=config['VALID_DATA_DIR'],
            target_size=config['IMAGE_SIZE'][:-1],
            batch_size=config['BATCH_SIZE'],
            class_mode='categorical')

        return training_set, valid_set

    else:
        train_datagen = ImageDataGenerator(rescale=1. / 255)
        valid_datagen = ImageDataGenerator(rescale=1. / 255)

        training_set = train_datagen.flow_from_directory(
            directory=config['TRAIN_DATA_DIR'],
            target_size=config['IMAGE_SIZE'][:-1],
            batch_size=config['BATCH_SIZE'],
            class_mode='categorical')

        valid_set = valid_datagen.flow_from_directory(
            directory=config['VALID_DATA_DIR'],
            target_size=config['IMAGE_SIZE'][:-1],
            batch_size=config['BATCH_SIZE'],
            class_mode='categorical')

        return training_set, valid_set


def class_name():
    train, valid = train_valid_generator()
    print(train.class_indices)




def manage_input_data(INPUT_IMAGE):
    '''
    This function takes an new raw image & convert the image
    into nd array with respect to training images dimention.
    :param INPUT_IMAGE:
    :return nd array:
    '''
    try:
        INPUT_IMAGE = INPUT_IMAGE
        test_image = image.load_img(INPUT_IMAGE, target_size= config['IMAGE_SIZE'][:-1])
        test_image = image.img_to_array(test_image)
        test_image = np.expand_dims(test_image, axis=0)

        return test_image
    except Exception as e:
        print(e)


'''
@Author: Bappy Ahmed
Date: 03 sep 2021
Email: entbappy73@gmail.com
'''

from flask import Flask, render_template, request,jsonify
from ImageSeeker.com_in_ineuron_ai_utils.utils import decodeImage
from flask_cors import CORS, cross_origin
from ImageSeeker.predict import dogcat
import json
import webbrowser
from threading import Timer



class ClientApp:
    def __init__(self):
        self.filename = "inputImage.jpg"
        self.classifier = dogcat(self.filename)



app = Flask(__name__) # initializing a flask app
CORS(app)

@app.route('/',methods=['GET'])  # route to display the home page
def homePage():
    return render_template("index.html")

@app.route('/input',methods=['GET'])  # route to display the home page
def input_form():
    return render_template("input_form.html")

@app.route('/train',methods=['POST','GET']) # route to show the predictions in a web UI
def train_func():
    if request.method == 'POST':
        try:
            # Data config
            TRAIN_DATA_DIR = request.form['TRAIN_DATA_DIR']
            VALID_DATA_DIR = request.form['VALID_DATA_DIR']
            CLASSES =int(request.form['CLASSES'])
            IMAGE_SIZE = request.form['IMAGE_SIZE']
            AUGMENTATION = request.form['AUGMENTATION']
            BATCH_SIZE = int(request.form['BATCH_SIZE'])


            # Model config
            MODEL_OBJ = request.form['MODEL_OBJ']
            MODEL_NAME = request.form['MODEL_NAME']
            FREEZE_ALL = request.form['FREEZE_ALL']
            OPTIMIZER = request.form['OPTIMIZER']
            EPOCHS = int(request.form['EPOCHS'])
            LOSS_FUNC = request.form['LOSS_FUNC']

            configs = {
                "TRAIN_DATA_DIR": TRAIN_DATA_DIR,
                "VALID_DATA_DIR": VALID_DATA_DIR,
                "AUGMENTATION": AUGMENTATION,
                "CLASSES": CLASSES,
                "IMAGE_SIZE": IMAGE_SIZE,
                "BATCH_SIZE": BATCH_SIZE,
                "MODEL_OBJ": MODEL_OBJ,
                "MODEL_NAME": MODEL_NAME,
                "EPOCHS": EPOCHS,
                "FREEZE_ALL": FREEZE_ALL,
                "OPTIMIZER": OPTIMIZER,
                "LOSS_FUNC": LOSS_FUNC

            }

            with open('config.json', 'w') as json_file:
                json.dump(configs, json_file)


            from ImageSeeker import train_engine

            hist = train_engine.train()

            return render_template('input_form.html', output = hist)

        except Exception as e:
            print('The Exception message is: ',e)
            return 'something is wrong'

    else:
        return render_template('index.html')


@app.route('/test',methods=['GET','POST'])  # route to display the home page
def predcit():
    return render_template("predict.html")


@app.route("/predict", methods=['POST'])
def predictRoute():
    clApp = ClientApp()
    image = request.json['image']
    decodeImage(image, clApp.filename)
    result = clApp.classifier.predictiondogcat()
    return jsonify(result)



def open_browser():
    webbrowser.open_new('http://127.0.0.1:8080/')


def start_app():
    Timer(1, open_browser).start()
    app.run(host="127.0.0.1", port=8080,debug=True)



if __name__ == "__main__":
    start_app()




import numpy as np
from tensorflow.keras.models import load_model
from tensorflow.keras.preprocessing import image

class dogcat:
    def __init__(self,filename):
        self.filename =filename


    def predictiondogcat(self):
        from ImageSeeker.utils import data_manager as dm
        from ImageSeeker.utils.config import configureModel
        from ImageSeeker.utils.config import configureData

        config_data = configureData()
        config_model = configureModel()

        # load model
        model_path = f"New_trained_model/{'new' + config_model['MODEL_NAME'] + '.h5'}"
        print('Loading...', model_path)
        model = load_model(model_path)

        # summarize model
        #model.summary()
        imagename = self.filename
        predict = dm.manage_input_data(imagename)
        result = model.predict(predict)
        results = np.argmax(result, axis=-1)
        print(dm.class_name())
        print(results)

        return [{ f"image class" : {str(results[0])}}]

        # if results[0] == 1:
        #     prediction = 'dog'
        #     return [{ "image" : prediction}]
        # else:
        #     prediction = 'cat'
        #     return [{ "image" : prediction}]





'''
@author: Bappy Ahmed
Email: entbappy73@gmail.com
Date: 06-sep-2021
'''



from ImageSeeker.utils import model
from ImageSeeker.utils import data_manager as dm
from ImageSeeker.utils.config import configureModel
from ImageSeeker.utils import callbacks
import tensorflow as tf

config_model = configureModel()


def train():


    """The logic for one training step.
  
    This method should contain the mathematical logic for one step of training.
    This typically includes the forward pass, loss calculation, backpropagation,
    and metric updates.

     Args:
      data: A nested structure of `Tensor`s.

    Returns:
      A `dict` containing values.Typically, the
      values of the `Model`'s metrics are returned. Example:
      `{'loss': 0.2, 'accuracy': 0.7}`.

    """
    model_obj = model.load_pretrain_model()
    my_model = model_obj
    train_data, valid_data = dm.train_valid_generator()

    #callbacks
    log_dir = callbacks.get_log_path()
    tb_cb = tf.keras.callbacks.TensorBoard(log_dir=log_dir)
    ckp = callbacks.checkpoint()

    call = [tb_cb, ckp]

    #Calculating steps_per_epoch & validation_steps
    steps_per_epoch = train_data.samples // train_data.batch_size
    validation_steps = valid_data.samples // valid_data.batch_size

    my_model.fit(
        train_data,
        validation_data=valid_data,
        epochs=config_model['EPOCHS'],
        steps_per_epoch=steps_per_epoch,
        validation_steps=validation_steps,
        callbacks=call
    )

    new_path = f"New_trained_model/{'new'+config_model['MODEL_NAME']+'.h5'}"
    my_model.save(new_path)
    print(f"Model saved at the following location : {new_path}")



from ImageSeeker.utils.config import configureData
from ImageSeeker.utils.config import configureModel
import tensorflow as tf
import os

config_data = configureData()
config_model = configureModel()

def get_model():
    """The logic for loading pretrain model.
  
     Args:
      MODEL_OBJ: Model object

    Returns:
      It returns keras model objcet

    """
    try:
       model =  config_model['MODEL_OBJ']
       print("Detected pretrain model!!")
       # os.makedirs('Models', exist_ok = True)
       # save_path = os.path.join('Models', config_model['MODEL_NAME']+'.h5')
       # print(f'Model has been saved following directory : {save_path}')
       # model.save(save_path)
       return model

    except Exception as e:
        print("Something went wrong!!", e)


def model_preparation(model):
    print('Preparing model...')
    if config_model['FREEZE_ALL'] == 'True':
        print('Freezing all...')
        for layer in model.layers:
            layer.trainable = False

        # Add custom layers
        flatten_in = tf.keras.layers.Flatten()(model.output)

        if config_data['CLASSES'] > 2:
            print('Adding softmax...')
            prediction = tf.keras.layers.Dense(
                units=config_data['CLASSES'],
                activation="softmax"
            )(flatten_in)

            full_model = tf.keras.models.Model(
                inputs=model.input,
                outputs=prediction
            )

            full_model.compile(
                optimizer=config_model['OPTIMIZER'],
                loss=config_model['LOSS_FUNC'],
                metrics=["accuracy"]
            )
            print('Model loaded!!')

            return full_model

        else:
            print('Adding sigmoid...')
            prediction = tf.keras.layers.Dense(
                units=config_data['CLASSES'],
                activation="sigmoid"
            )(flatten_in)

            full_model = tf.keras.models.Model(
                inputs=model.input,
                outputs=prediction
            )

            full_model.compile(
                optimizer=config_model['OPTIMIZER'],
                loss=config_model['LOSS_FUNC'],
                metrics=["accuracy"]
            )

            print('Model loaded!!')

            return full_model


    else:

        for layer in model.layers[:config_model['FREEZE_TILL']]:
            layer.trainable = False

            # Add custom layers
            flatten_in = tf.keras.layers.Flatten()(model.output)

            if config_data['CLASSES'] > 2:
                prediction = tf.keras.layers.Dense(
                    units=config_data['CLASSES'],
                    activation="softmax"
                )(flatten_in)

                full_model = tf.keras.models.Model(
                    inputs=model.input,
                    outputs=prediction
                )

                full_model.compile(
                    optimizer=config_model['OPTIMIZER'],
                    loss=config_model['LOSS_FUNC'],
                    metrics=["accuracy"]
                )
                print('Model loaded!!')

                return full_model

            else:
                prediction = tf.keras.layers.Dense(
                    units=config_data['CLASSES'],
                    activation="sigmoid"
                )(flatten_in)

                full_model = tf.keras.models.Model(
                    inputs=model.input,
                    outputs=prediction
                )

                full_model.compile(
                    optimizer=config_model['OPTIMIZER'],
                    loss=config_model['LOSS_FUNC'],
                    metrics=["accuracy"]
                )

                print('Model loaded!!')

                return full_model



def load_pretrain_model():

    """The logic for loading pretrain model.
  
     Args:
      MODEL_OBJ: Model object

    Returns:
      It returns keras model objcet

    """
    model = get_model()
    model = model_preparation(model)
    return model


def load_exist_model():

    """The logic for loading an existing model.
  
     Args:
      PRETRAIN_MODEL_DIR: Your existing model path

    Returns:
      It returns keras model objcet

    """
    print('Loading existing model...')
    print("Model loaded!")
    model = tf.keras.models.load_model(config_model['PRETRAIN_MODEL_DIR'])
    model = model_preparation(model)
    return model













from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
  name = 'ImageSeeker',
  packages = find_packages(),
  include_package_data=True,
  version = '0.3',
  license='MIT',
  description = 'Its an auto image classification library',
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/entbappy/ImageSeeker-Package",
  author = 'Bappy Ahmed',
  author_email = 'entbappy73@gmail.com',
  keywords = ['imageseeker'],
  install_requires=[
        'tensorflow==2.4.1',
        'scipy==1.6.3',
        'numpy',
        'pandas',
        'Pillow==8.3.2',
        'Flask',
        'Flask-Cors==3.0.10'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',

    # Indicate who your project is intended for
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',

    # Pick your license as you wish
    'License :: OSI Approved :: MIT License',

    # Specify the Python versions you support here. In particular, ensure
    # that you indicate whether you support Python 2, Python 3 or both.
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
  entry_points={
        "console_scripts": [
            "imgseeker = ImageSeeker.main:start_app",
        ]},
)
